<?php

/**
 * i-doit
 *
 * Auth: Class for CMDB module authorization rules.
 *
 * @package     i-doit
 * @subpackage  auth
 * @author      Selcuk Kekec <skekec@i-doit.com>
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 */
class isys_auth_cmdb_objects extends
    isys_auth_cmdb
{
    /**
     * Container for singleton instance
     *
     * @var isys_auth_cmdb_objects
     */
    private static $m_instance = NULL;


    /**
     * Gets all allowed objects ordered by object types which will be cached in the auth cache file.
     *
     * @return  mixed
     * @author  Van Quyen Hoang <qhoang@i-doit.com>
     */
    public function get_allowed_objects()
    {
        // Check for inactive auth system
        if (!$this->is_auth_active())
        {
            return true;
        } // if

        global $g_comp_database, $g_comp_session;

        $l_return   = $l_object_types = array();
        $l_wildcard = false;

        $l_cache_obj = isys_caching::factory('auth-' . $g_comp_session->get_user_id());
        $l_cache     = $l_cache_obj->get('allowed_objects');

        if ($l_cache === false || (is_array($l_cache) && count($l_cache) == 0))
        {
            /** @var isys_cmdb_dao $l_dao */
            $l_dao = isys_factory_cmdb_dao::get_instance('isys_cmdb_dao', $g_comp_database);

            // Get object types from object in type rights
            if (isset($this->m_paths['obj_in_type']))
            {
                if (isset($this->m_paths['obj_in_type'][isys_auth::WILDCHAR]))
                {
                    $l_wildcard = true;
                    $l_return   = true;
                }
                else
                {
                    if (!isset($this->m_paths['obj_in_type'][isys_auth::EMPTY_ID_PARAM]))
                    {
                        foreach ($this->m_paths['obj_in_type'] AS $l_key => $l_rights)
                        {
                            $l_key = strtoupper($l_key);
                            if (defined($l_key))
                            {
                                $l_key_constant                  = constant($l_key);
                                $l_object_types[$l_key_constant] = $l_key_constant;
                            } // if
                        } // foreach
                    } // if
                } // if
            } // if

            // Get object types from object id rights
            if (!$l_wildcard && isset($this->m_paths['obj_id']))
            {
                if (isset($this->m_paths['obj_id'][isys_auth::WILDCHAR]))
                {
                    $l_wildcard = true;
                    $l_return   = true;
                }
                else
                {
                    if (!isset($this->m_paths['obj_id'][isys_auth::EMPTY_ID_PARAM]))
                    {
                        foreach ($this->m_paths['obj_id'] AS $l_key => $l_rights)
                        {
                            $l_type_id                    = $l_dao->get_objTypeID($l_key);
                            $l_return[$l_type_id][$l_key] = $l_key;
                        } // foreach
                    } // if
                } // if
            } // if

            // Get object types from location rights
            if (!$l_wildcard && isset($this->m_paths['location']))
            {
                if (isset($this->m_paths['location'][isys_auth::WILDCHAR]))
                {
                    $l_wildcard = true;
                    $l_return   = true;
                }
                else
                {
                    if (!isset($this->m_paths['location'][isys_auth::EMPTY_ID_PARAM]))
                    {
                        // The given ID could not be found directly, now we check the location path.

                        /** @var isys_cmdb_dao_location $l_dao_location */
                        $l_dao_location = isys_factory_cmdb_dao::get_instance('isys_cmdb_dao_location',
                                                                              $g_comp_database);

                        foreach ($this->m_paths['location'] as $l_location_id => $l_rights)
                        {
                            // Get child locations of the location auth-paths.
                            $l_child_locations = array_keys($l_dao_location->get_child_locations_recursive($l_location_id));
                            foreach ($l_child_locations AS $l_obj_id)
                            {
                                $l_objtype_id = $l_dao_location->get_objTypeID($l_obj_id);
                                if (!isset($l_object_types[$l_objtype_id]))
                                {
                                    $l_return[$l_objtype_id][$l_obj_id] = $l_obj_id;
                                } // if
                            } // foreach
                        } // foreach
                    } // if
                } // if
            } // if

            if (count($l_object_types) > 0 && !$l_wildcard)
            {
                $l_res = $l_dao->get_objects_by_type(array_keys($l_object_types));

                // $l_return needs to be an array here!
                $l_return = (is_array($l_return)) ? $l_return : array();

                while ($l_row = $l_res->get_row())
                {
                    $l_return[$l_row['isys_obj__isys_obj_type__id']][$l_row['isys_obj__id']] = $l_row['isys_obj__id'];
                } // while
            } // if

            if (!$l_wildcard)
            {
                $l_return = (count($l_return) > 0) ? $l_return : false;
            } // if

            try
            {
                $l_cache_obj->set('allowed_objects', $l_return)->save();
            }
            catch (isys_exception_filesystem $e)
            {
                isys_notify::warning($e->getMessage());
            }
        }
        else
        {
            $l_return = $l_cache;
        } // if

        return $l_return;
    } // function


    /**
     * Gets all or by object type allowed objects.
     *
     * @param   integer $p_type_id
     *
     * @return  mixed
     * @author  Van Quyen Hoang <qhoang@i-doit.com>
     */
    public function get_allowed_objects_by_type($p_type_id = NULL)
    {
        // Check for inactive auth system
        if (!$this->is_auth_active())
        {
            return true;
        } // if

        $l_allowed_objects = $this->get_allowed_objects();

        if (is_array($l_allowed_objects))
        {
            if (!empty($p_type_id))
            {
                if (isset($l_allowed_objects[$p_type_id]))
                {
                    return array_keys($l_allowed_objects[$p_type_id]);
                } // if
            }
            else
            {
                if (count($l_allowed_objects) > 0)
                {
                    $l_return = array();
                    foreach ($l_allowed_objects as $l_obj_content)
                    {
                        $l_return = array_merge($l_return, array_keys($l_obj_content));
                    } // foreach

                    return $l_return;
                }
                else
                {
                    // In case of wildcard.
                    return true;
                } // if
            } // if
        }
        else
        {
            if ($l_allowed_objects === true)
            {
                return true;
            } // if
        } // if

        return false;
    } // function

    /**
     * Retrieve singleton instance of authorization class
     *
     * @return isys_auth_cmdb_objects
     * @author Selcuk Kekec <skekec@i-doit.com>
     */
    public static function instance()
    {
        // If the DAO has not been loaded yet, we initialize it now.
        if (self::$m_dao === NULL)
        {
            global $g_comp_database;

            self::$m_dao = new isys_auth_dao($g_comp_database);
        } // if

        if (self::$m_instance === NULL)
        {
            self::$m_instance = new self;
        } // if

        return self::$m_instance;
    } // function
} // class