<?php

/**
 * @package     i-doit
 * @subpackage  General
 * @author      Dennis Stücken <dstuecken@i-doit.org>
 * @version     1.6
 * @copyright   synetics GmbH
 * @license     http://www.gnu.org/licenses/agpl-3.0.html GNU AGPLv3
 */
class isys_cache_fs
    extends isys_cache_keyvalue
    implements isys_cache_keyvaluable
{
    /**
     * Cache directory.
     *
     * @var  string
     */
    private $m_directory = 'temp/fscache/';

    /**
     * Cache expiration time in seconds
     *
     * @var int
     */
    protected $m_options = array(
        'expiration' => 60
    );

    /**
     * Set $p_key to $p_value in filesystem persistent cache.
     *
     * @param   string  $p_key
     * @param   mixed   $p_value
     * @param   integer $p_ttl
     *
     * @return  isys_cache_fs
     */
    public function set($p_key, $p_value = NULL, $p_ttl = -1)
    {
        $this->prepend_ns($p_key);

        file_put_contents($this->m_directory . md5($p_key), serialize($p_value));

        return $this;
    } // function

    /**
     * Flush cache
     *
     * @return boolean
     */
    public function flush()
    {
        if (is_dir($this->m_directory))
        {
            $iterator = new DirectoryIterator($this->m_directory);
            foreach ($iterator as $file)
            {
                if ($file->isFile() && $file->isWritable())
                {
                    unlink($file->getPathname());
                }
            }

            return true;
        }

        return false;
    }

    /**
     * Get value of $p_key from filesystem.
     *
     * @param   string $p_key
     *
     * @return  mixed
     */
    public function get($p_key)
    {
        $this->prepend_ns($p_key);

        $l_hash = md5($p_key);

        return
            file_exists($this->m_directory . $l_hash) && (filemtime($this->m_directory . $l_hash) > (time() - (int) @$this->m_options['expiration']))
                ? unserialize(file_get_contents($this->m_directory . $l_hash)) : NULL;
    } // function

    /**
     * Determine whether a storage entry has been set for a key.
     *
     * @param   string $key The storage entry identifier.
     *
     * @return  boolean
     */
    public function exists($key)
    {
        $this->prepend_ns($p_key);

        $l_hash = md5($p_key);

        return file_exists($this->m_directory . $l_hash) && (filemtime($this->m_directory . $l_hash) > (time() - (int) @$this->m_options['expiration']));
    }

    /**
     * Deletes a cache item from filesystem.
     *
     * @param   string $p_key
     *
     * @return  isys_cache_fs
     */
    public function delete($p_key)
    {
        $this->prepend_ns($p_key);

        $l_hash = md5($p_key);

        if (file_exists($l_hash))
        {
            unlink($l_hash);
        } // if

        return $this;
    } // function

    /**
     * Check wheather fs cache is available or not.
     *
     * @return  boolean
     */
    public static function available()
    {
        return true;
    } // function

    /**
     * Constructor.
     */
    public function __construct()
    {
        $this->m_directory = rtrim(isys_application::instance()->app_path, '/') . '/' . $this->m_directory;

        if (!is_dir($this->m_directory))
        {
            if (is_writable(dirname($this->m_directory)))
            {
                mkdir($this->m_directory);
            }
            else
            {
                throw new isys_exception_cache('Filesystem cache not available. Could not write to: ' . $this->m_directory, 'fs');
            } // if
        } // if
    } // function
} // class