"use strict";

/**
 * i-doit javascript base class for the new TableComponent.
 *
 * @author  Leonard Fischer <lfischer@i-doit.com>
 */
window.TableComponent = Class.create({
    initialize: function ($table, options) {
        this.$table = $table;

        if (!this.$table.down('.dynamic-replacement'))
        {
            this.progressTablePlaceholders();
        }

        this.$cols = this.$table.down('colgroup').select('col');
        this.$head = this.$table.down('thead');
        this.$body = this.$table.down('tbody');
        this.$header = this.$head.select('th');
        this.$dragger = new Element('div', {className: 'mainTableDragger hide'});
        this.$lastClickedCheckbox = null;
        this.$ghost = null;

        this.options = {
            dragDrop:               false,
            dragDropBoxContent:     '%count% object(s)',
            checkboxes:             false,
            resizeColumns:          false,
            resizeColumnAjaxURL:    '',
            columnSizes:            [],
            order:                  false,
            orderColumns:           [],
            currentOrderColumn:     '',
            currentOrderDirection:  '',
            rowClick:               false,
            rowClickUrl:            '?objID=%id%',
            keyboardCommands:       false,
            orderAscendingMessage:  '',
            orderDescendingMessage: '',
            loadingMessage:         ''
        };

        Object.extend(this.options, options || {});

        if (this.options.resizeColumns && this.options.resizeColumnAjaxURL)
        {
            this.prepareColumnDrag();
        }

        if (this.options.dragDrop)
        {
            this.prepareDragDrop();
        }

        if (this.options.order)
        {
            this.prepareTableOrdering();
        }

        if (!Object.isArray(this.options.columnSizes))
        {
            this.options.columnSizes = [];
        }

        // If the number of defined column sizes is the same as the number of our headers... Go for it!
        if (this.options.columnSizes.length === (this.$header.length - (this.options.checkboxes ? 1 : 0)))
        {
            this.setInitialColumnWidths(this.options.columnSizes);
        }
        else
        {
            this.resetColumns();
        }

        this.callbackManager = new window.TableComponentCallbackManager($table, options);

        // Prepare the observer AFTER the TableComponentCallbackManager (because of "that.options.rowClick").
        this.prepareObserver();
    },

    setInitialColumnWidths: function (columnSizes) {
        for (var i in columnSizes)
        {
            if (columnSizes.hasOwnProperty(i))
            {
                if (!this.$cols.hasOwnProperty(i))
                {
                    continue;
                }

                this.$cols[i].setStyle({width: columnSizes[i]});
            }
        }
    },

    progressTablePlaceholders: function () {
        this.$table.innerHTML = window.replaceTablePlaceholders(this.$table.innerHTML);
    },

    prepareColumnDrag: function () {
        var that = this, i;

        for (i in that.$header)
        {
            if (!that.$header.hasOwnProperty(i))
            {
                continue;
            }

            if (that.options.checkboxes && i == 0)
            {
                continue;
            }

            if (i == (that.$header.length - 1))
            {
                continue;
            }

            that.$header[i].insert(new Element('div', {className: 'handle'}));
        }

        that.$table.insert({after: that.$dragger});
    },

    prepareDragDrop: function () {
        this.$ghost = new Element('div', {className: 'mainTableDragGhost box-white', 'data-table-id': this.$table.id}).hide();

        $('body').insert(this.$ghost);
    },

    prepareTableOrdering: function () {
        var that = this, i;

        for (i in that.$header)
        {
            var currentOrder;

            if (!that.$header.hasOwnProperty(i) || !that.$header[i].down('.sort-arrow'))
            {
                continue;
            }

            if (that.options.orderColumns.hasOwnProperty((i-1)))
            {
                currentOrder = (that.options.orderColumns[(i-1)] == that.options.currentOrderColumn);

                that.$header[i].down('.sort-arrow')
                    .writeAttribute({
                       src:       window.dir_images + 'icons/silk/bullet_arrow_' + ((currentOrder && that.options.currentOrderDirection == 'DESC') ? 'down' : 'up') + '.png',
                       className: 'sort-arrow' + (currentOrder ? '' : ' opacity-30'),
                       title:     (currentOrder && that.options.currentOrderDirection == 'DESC' ? that.options.orderAscendingMessage : that.options.orderDescendingMessage)
                    })
                    .next('.overflowable').addClassName('mouse-pointer')
                    .writeAttribute({
                       'data-action':          'order-by-column',
                       'data-order-field':     that.options.orderColumns[(i-1)],
                       'data-order-direction': (currentOrder ? that.options.currentOrderDirection : 'ASC')
                    });
            }
        }
    },

    prepareObserver: function () {
        var that = this, i;

        that.$table.on('remove:entriesByIds', function (ev) {
            if (!Object.isArray(ev.memo.ids))
            {
                return;
            }

            that.$body.select('tr[data-id]')
                .filter(function ($tr) {
                    return ev.memo.ids.in_array($tr.readAttribute('data-id'));
                })
                .invoke('remove');
        });

        if (that.options.keyboardCommands)
        {
            document.on('keydown', function (ev) {
                var $currentRow = that.$body.down('tr.highlightLine'),
                    $checkbox;

                if ($currentRow)
                {
                    $checkbox = $currentRow.down('.mainTableCheckbox input');
                }

                if (!['input', 'textarea', 'select', 'a', 'button'].in_array(document.activeElement.tagName.toLowerCase()))
                {
                    if ([13, 32, 38, 40].in_array(ev.keyCode))
                    {
                        ev.preventDefault();
                    }

                    switch (ev.keyCode)
                    {
                        // Up
                        case 38:
                            if ($currentRow.previous('tr'))
                            {
                                $currentRow.removeClassName('highlightLine').previous('tr').addClassName('highlightLine');
                            }

                            break;

                        // Down
                        case 40:
                            if ($currentRow.next('tr'))
                            {
                                $currentRow.removeClassName('highlightLine').next('tr').addClassName('highlightLine');
                            }

                            break;

                        // Spacebar
                        case 32:
                            if (that.$lastClickedCheckbox !== null && window.keyPressedShift)
                            {
                                that._selectRange($checkbox, that.$lastClickedCheckbox)
                                    .invoke('setValue', $checkbox.checked ? 1 : 0)
                                    .invoke('simulate', 'change');
                            }

                            $checkbox.simulate('click');

                            break;

                        // Enter
                        case 13:
                            $checkbox.setValue('1').simulate('click');

                            // @todo  Check if we should control this via parameters.
                            $('navbar_item_C__NAVMODE__EDIT').simulate('click');
                            break;
                    }
                }
            });

            that.$body.down('tr').addClassName('highlightLine');
        }

        if (that.options.rowClick)
        {
            // We use the CSS classes to "show" the user, the rows are clickable.
            that.$body.addClassName('mouse-pointer');

            that.$body.on('click', 'tr', function (ev) {
                var $tr = ev.findElement('tr'),
                    $td = ev.findElement('td');

                if (that.options.checkboxes && $tr.down('td') === $td)
                {
                    // Clicking the checkbox TD will not open the object.
                    return;
                }

                document.location.href = that.options.rowClickUrl.replace('%id%', $tr.readAttribute('data-id'));
            });
        }

        if (that.options.checkboxes)
        {
            that.$head.on('click', 'label.mainTableCheckbox input', function (ev) {
                var $checkbox = ev.findElement('input');

                that.$body.select('label.mainTableCheckbox input')
                    .invoke('setValue', $checkbox.checked ? 1 : 0)
                    .invoke('simulate', 'change');

                that.$lastClickedCheckbox = null;
            });

            that.$body.on('click', 'label.mainTableCheckbox input', function (ev) {
                var $checkbox = ev.findElement('input');

                if (that.$lastClickedCheckbox !== null && window.keyPressedShift)
                {
                    that._selectRange($checkbox, that.$lastClickedCheckbox)
                        .invoke('setValue', $checkbox.checked ? 1 : 0)
                        .invoke('simulate', 'change');
                }

                that.$lastClickedCheckbox = $checkbox;
            });

            that.$body.on('change', 'label.mainTableCheckbox input', function (ev) {
                var $checkbox = ev.findElement('input');

                $checkbox.up('tr')[$checkbox.checked ? 'addClassName' : 'removeClassName']('tr-selected');
            });

            if (that.options.keyboardCommands)
            {
                // This helps the keyboard navigation, after klicking any input field.
                that.$body.on('mouseup', 'input', function (ev) {
                    ev.preventDefault();
                    ev.findElement('input').blur();
                });
            }
        }

        if (that.options.resizeColumns && that.options.resizeColumnAjaxURL)
        {
            var $handles = that.$header.invoke('down', '.handle');

            for (i in $handles)
            {
                if (!$handles.hasOwnProperty(i) || !$handles[i])
                {
                    continue;
                }

                new Draggable($handles[i], {
                    constraint: 'horizontal',
                    onStart:    that._columnDragStart.bind(that),
                    onDrag:     that._columnDrag.bind(that),
                    onEnd:      that._columnDragEnd.bind(that)
                });
            }
        }

        if (this.options.dragDrop)
        {
            var $tr = that.$body.select('tr');

            for (i in $tr)
            {
                if (!$tr.hasOwnProperty(i))
                {
                    continue;
                }

                new Draggable($tr[i], {
                    handle:   $tr[i].down('.drag-handle'),
                    revert:   true,
                    ghosting: true,
                    zindex:   3,
                    onStart:  that._rowDragStart.bind(that),
                    onDrag:   that._rowDrag.bind(that),
                    onEnd:    that._rowDragEnd.bind(that)
                });
            }
        }

        var $quickinfo = this.$body.select('.quickinfo');

        // Prepare the quickinfo.
        for (i in $quickinfo)
        {
            if (!$quickinfo.hasOwnProperty(i))
            {
                continue;
            }

            new Tip($quickinfo[i], '', {
                ajax:      {url: window.www_dir + '?ajax=1&call=quick_info&objID=' + $quickinfo[i].readAttribute('data-object-id')},
                delay:     '0',
                stem:      'topLeft',
                style:     'default',
                className: 'objectinfo'
            });
        }
    },

    _selectRange: function ($checkboxA, $checkboxB) {
        var inRange = false;

        return this.$body.select('label.mainTableCheckbox input').filter(function ($checkbox) {
            if (!inRange && $checkbox !== $checkboxA && $checkbox !== $checkboxB)
            {
                return false;
            }
            else if ($checkbox === $checkboxA || $checkbox === $checkboxB)
            {
                inRange = !inRange;
                return true;
            }
            else if (inRange)
            {
                return true;
            }
        })
    },

    _columnDragStart: function (drag) {
        var that    = this,
            $handle = drag.element,
            eOffset = $handle.viewportOffset(),
            pOffset = that.$table.viewportOffset(),
            offset  = eOffset.relativeTo(pOffset);

        that.$dragger.removeClassName('hide')
            .setStyle({
                position: 'absolute',
                top:      '61px',
                left:     (6 + offset.left) + 'px',
                height:   that.$table.getHeight() + 'px'
            });

        that.$table.store('leftOriginWidth', drag.element.up('th').getWidth())
            .store('rightOriginWidth', drag.element.up('th').next('th').getWidth());
    },

    _columnDrag: function (drag) {
        var that = this, left = drag.element.getStyle('left');

        if (parseInt(left) < 30)
        {
            // The user shall not be able to shrink his table to 0.
            return;
        }

        var $th     = drag.element.up('th'),
            $thNext = $th.next('th');

        if (parseInt(left) > ($th.getWidth() + $thNext.getWidth() - 30))
        {
            // The user shall not be able to grow his table above 100.
            return;
        }

        that.$table.store('delta', parseInt(left));

        var $handle = drag.element,
            eOffset = $handle.viewportOffset(),
            pOffset = that.$table.viewportOffset(),
            offset  = eOffset.relativeTo(pOffset);

        that.$dragger.setStyle({left: (6 + offset.left) + 'px'});
    },

    _columnDragEnd: function (drag) {
        var that = this;

        that._setColumnWidth(parseInt(that.$header.indexOf(drag.element.up('th'))));

        that.$dragger.addClassName('hide');

        drag.element
            .setStyle({
                left:   null,
                right:  '-6px',
                zIndex: 100
            });

        delay(function () {
            that.saveColumnWidths();
        }, 1000);
    },

    _rowDragStart: function (drag) {
        var draggedIDs, that = this;

        // @todo  Handle "that.options.checkboxes".

        // Hide the dragged row (ghost) and check the checkbox.
        drag.element.addClassName('draggable').addClassName('hide').previous().down('label.mainTableCheckbox input').setValue(1);

        // Add a additional "dummy" row, to keep the "stripes" in place.
        drag.element.insert({before: new Element('tr', {className: 'dummy'})});

        // Store the information, if the checkbox was checked, before.
        drag.element.store('wasChecked', drag.element.down('label.mainTableCheckbox input').checked).down('label.mainTableCheckbox input').setValue(1);

        // Now get all the checked checkboxes and filter duplicates (because of the ghosting stuff).
        draggedIDs = that.$body.select('label.mainTableCheckbox input:checked').invoke('getValue').uniq();

        that.$ghost
            .writeAttribute('data-ids', draggedIDs.join())
            .update(this.options.dragDropBoxContent.replace('%count%', draggedIDs.length))
            .appear({duration: 0.5});
    },

    _rowDrag: function () {
        // Move our ghost around.
        this.$ghost
            .setStyle({
                left: (12 + window.mouseX) + 'px',
                top:  window.mouseY + 'px'
            });
    },

    _rowDragEnd: function (drag) {
        // Display the row (ghost) again and remove the dummy.
        drag.element.removeClassName('draggable').removeClassName('hide').previous('.dummy').remove();

        // If the checkbox was not checked, uncheck it.
        if (!drag.element.retrieve('wasChecked'))
        {
            drag.element.down('label.mainTableCheckbox input').setValue(0);
        }

        // Hide our ghost.
        this.$ghost.fade({duration: 0.5});
    },

    saveColumnWidths: function () {
        var that = this;

        new Ajax.Request(this.options.resizeColumnAjaxURL, {
            parameters: {columns: Object.toJSON(that.$cols.invoke('getStyle', 'width'))},
            onSuccess:  function (xhr) {
                var json = xhr.responseJSON;

                if (!json.success)
                {
                    idoit.Notify.error(json.message, {sticky: true});
                }
            }
        });
    },

    _setColumnWidth: function (index) {
        var leftOriginWidth = this.$table.retrieve('leftOriginWidth'),
            rightWidth      = this.$table.retrieve('rightOriginWidth'),
            delta           = this.$table.retrieve('delta'),
            sumWidth        = (leftOriginWidth + rightWidth) - 18; // Somehow the (left + right) TH will expand slightly with every iteration... So we subtract.

        // Left column.
        this.$cols[index].setStyle({width: delta + 'px'});

        // Right column
        this.$cols[index + 1].setStyle({width: (sumWidth - delta) + 'px'});

        this.$table.store('leftOriginWidth', null).store('rightOriginWidth', null).store('delta', null);
    },

    resetColumns: function () {
        var cols = this.$cols.length;

        if (this.options.checkboxes)
        {
            --cols;
        }

        this.$cols.invoke('setStyle', {width: (100 / cols) + '%'});

        if (this.options.checkboxes)
        {
            this.$cols[0].setStyle({width: '40px'});
        }

        this.saveColumnWidths();
    }
});

window.replaceTablePlaceholders = function (html) {
    // Replace the "Title {1}" with the root location house.
    html = html.replace(/[\w-]+ \{1\}/g, '<img class="vam" src="' + window.dir_images + 'icons/silk/house.png">');

    // Replace object links with format "Title {id}".
    html = html.replace(/([^\{\>,]+) \{([0-9]+)\}/g, ' <a class="dynamic-replacement quickinfo" href="?objID=$2" data-object-id="$2">$1</a>');

    // Replace "{#123456}" with the cmdb-marker.
    return html.replace(/\{(#[0-9a-fA-F]{3,6})\}/g, '<div class="dynamic-replacement cmdb-marker" style="background-color: $1;"></div>');
};