/*
 * Lightweight Prototype based JavaScript tree view consuming Ajax datasources.
 */

/*
 * Initialize tree view with the given name(an empty element with
 * the id must exist). The data source url delivers the data for the
 * tree view. The placeholder {0} is replaced by the id.
 */
function eTree (treeName, treeDataSource, treeCaptionIcon, treeCaption, selectedNodes, imagesFolder, nodeCssClass, selectedNodeCssClass, ajaxAnimation, highlightLocationTypes) {
	// Store mandatory parameters.
	this.treeName = treeName;
	this.treeDataSource = treeDataSource;

	// Store optional parameters.
	this.treeCaptionIcon = treeCaptionIcon ? treeCaptionIcon : "";
	this.treeCaption = treeCaption ? treeCaption : "";
	this.imagesFolder = imagesFolder ? imagesFolder : "";
	this.nodeCssClass = nodeCssClass ? nodeCssClass : "node";
	this.selectedNodeCssClass = selectedNodeCssClass ? selectedNodeCssClass : "nodeSel";
	this.ajaxAnimation = ajaxAnimation ? ajaxAnimation : this.imagesFolder + "../please_wait.gif";
	this.highlightLocationTypes = (Object.isUndefined(highlightLocationTypes) ? false : highlightLocationTypes);

	// Expand nodes to expand into an array in reverse order.
	this.selectedNodesArray = selectedNodes.split(",");

	// Set the previously selected node.
	this.prevSelectedNodeId = -1;

	// Nodes who's child nodes have been loaded.
	this.loadedNodes = [];

	// Tree levels of nodes.
	this.treeLevels = [];

	// Nodes that are expanded.
	this.expandedNodes = [];

	// Nodes that mark the bottom of their level.
	this.expandedNodeBottom = [];

	// Node hierarchy
	this.parentNodes = [];

	// Image replaced by the tree expansion.
	this.expandingImage = null;

	// These callbacks will be triggered by the corresponding action.
	this.callback_after_render = this.callback_before_render = this.callback_after_load = this.callback_before_load = Prototype.emptyFunction;

	// Load tree root.
	this.LoadTree(-1);
}

// Load a node. A parent id of -1 indicates, that the root of the tree is loaded.
eTree.prototype.LoadTree = function (parentNodeId) {
	// Obtain expander icon and exchange it with animation.
	var expander = $(this.treeName + "_Expand_" + parentNodeId);

	this.callback_before_load(parentNodeId);

	if (expander) {
		this.expandingImage = this.imagesFolder + this.GetExpanderImage(this.expandedNodeBottom [parentNodeId], true);
		expander.src = this.ajaxAnimation;
	}

	// Build url for Ajax request.
	var url = this.treeDataSource.replace(/\{0\}/, parentNodeId);

	// Create Ajax request.
	new Ajax.Request(
		url, {
			method: 'post',
			parameters: $('isys_form').serialize(true),
			onSuccess: function (transport) {
				this.callback_after_load(parentNodeId);
				this.LoadTreeComplete(transport.responseJSON, parentNodeId);
			}.bind(this),
			onFailure: function () {}
		});
};

// Callback invoked, when data has arrived.
eTree.prototype.LoadTreeComplete = function (treeData, parentNodeId) {

	// Show expander
	var expander = $(this.treeName + "_Expand_" + parentNodeId), treeLevel, parentNode;
	if (expander) {
		if (this.expandingImage === null) {
			this.expandingImage = this.imagesFolder + this.GetExpanderImage(true, true, false);
		}

		expander.src = this.expandingImage;
		this.expandingImage = null;
	}

	// Locate parent node in dom and set the tree level.
	if (parentNodeId == -1) {
		// Tree root.
		parentNode = $(this.treeName);
		treeLevel = 0;

		// If applicable, create caption.
		if (this.treeCaptionIcon != "" || this.treeCaption != "") {
			var captionDiv = new Element("div", {className:'node-div root', 'data-object-id':parentNodeId});

			if (this.treeCaptionIcon != '') {
				captionDiv.insert(new Element("img", {src: this.treeCaptionIcon, className:'mr5'}));
			}

			if (this.treeCaption != '') {
				captionDiv.insert(this.treeCaption);
			}

			parentNode.insert(captionDiv);
		}
	} else {
		parentNode = $(this.treeName + "_ChildNodes_" + parentNodeId);
		treeLevel = this.treeLevels [parentNodeId];
	}

	this.callback_before_render(parentNodeId, parentNode);

	// Iterate over sub nodes.
	for (var i = 0; i < treeData.length; i++) {
		// Store node hierarchy.
		if (parentNodeId != -1) {
			this.parentNodes [treeData [i].id] = parentNodeId;
		}

		// Construct a div that contains:
		var nodeDiv = new Element("div", {id: this.treeName + "_Node_" + treeData[i].id, className:'node-div', 'data-object-id':treeData[i].id});
		var bottomNodes = [], nodeId = parentNodeId;

		if (this.highlightLocationTypes) {
			if (treeData[i].is_logically_assigned) {
				nodeDiv.insert(new Element('div', {className:'cmdb-marker', style:'background:#FFB85C;'}));
			} else if (treeData[i].is_physically_assigned) {
				nodeDiv.insert(new Element('div', {className:'cmdb-marker', style:'background:#66C1FF;'}));
			}
		}

		for (var k = 0; k < treeLevel; k++) {
			bottomNodes.push(nodeId);
			if (this.parentNodes [nodeId]) nodeId = this.parentNodes [nodeId];
			else break;
		}
		bottomNodes.pop();

		// 1) Spacer images.
		for (var j = 0, k = bottomNodes.length - 1; j < treeLevel; j++, k--) {
			// Determine, whether the parent at this level is the last sibling.
			var isLastSibling = this.expandedNodeBottom [bottomNodes [k]] == true;

			var levelSpacer = new Element("img");
			levelSpacer.setAttribute("src", this.imagesFolder + this.GetSpacerIcon(isLastSibling, j, treeLevel, i, treeData.length));

			nodeDiv.insert(levelSpacer);
		}

		// 2) Expansion icon.
		var expansionIcon = new Element("img", {id: this.treeName + "_Expand_" + treeData [i].id});

		// Only wire up an onclick for non leaf nodes.
		if (!treeData [i].is_leaf) {
			expansionIcon.onclick = this.ExpandCollapse.bind(this, treeData [i].id);

			if (i == treeData.length - 1) {
				this.expandedNodeBottom [treeData [i].id] = true;
				expansionIcon.src = this.imagesFolder + this.GetExpanderImage(true, false, false);
			} else {
				this.expandedNodeBottom [treeData [i].id] = false;
				expansionIcon.src = this.imagesFolder + this.GetExpanderImage(false, false, false);
			}
		} else {
			expansionIcon.src = this.imagesFolder + this.GetExpanderImage(false, false, true);
		}

		nodeDiv.insert(expansionIcon);

		// 3) Node image.
		nodeDiv.insert(new Element("img", {className:'nodeIcon', src: treeData[i].icon ? treeData[i].icon : this.imagesFolder + "page.gif"}));

		// 4) Node link.
		var nodeLink = new Element("a", {
			id: this.treeName + "_Link_" + treeData[i].id,
			onClick: treeData[i].url,
			className: this.nodeCssClass
		});

		nodeDiv.insert(nodeLink.insert(treeData[i].text));

		// 5) Child nodes container.
		var childNodeDiv = new Element("div", {id: this.treeName + "_ChildNodes_" + treeData[i].id});

		parentNode.insert(nodeDiv.insert(childNodeDiv));

		// Store node's tree level.
		this.treeLevels[treeData[i].id] = treeLevel + 1;
	}

	// Tag loaded / expanded node.
	this.loadedNodes[parentNodeId] = true;
	this.expandedNodes[parentNodeId] = true;

	// When loading a level has finished, continue on working with the selected nodes.
	if (this.selectedNodesArray.length > 1) {
		var expandNode = this.selectedNodesArray.pop();
		this.LoadTree(expandNode);
	} else {
		// Select the given node.
		this.Select(this.selectedNodesArray.pop());
	}

	this.callback_after_render(parentNodeId, (parentNodeId == -1) ? parentNode : $(this.treeName + "_Node_" + parentNodeId));
};

// Perform expansion / collapse of a node.
eTree.prototype.ExpandCollapse = function (nodeId) {
	// If an expansion already happened, do nothing.
	if (this.expandingImage != null) {
		return;
	}

	// Obtain expander icon and toggle it's display.
	var expander = $(this.treeName + "_Expand_" + nodeId);

	expander.src = this.imagesFolder + this.GetExpanderImage(this.expandedNodeBottom [nodeId], !this.expandedNodes [nodeId]);

	if (this.expandedNodes [nodeId]) {
		// Node is expanded, perform a collapse.
		$(this.treeName + "_ChildNodes_" + nodeId).hide();
		this.expandedNodes [nodeId] = false;
	} else {
		// If the node's child items where not loaded yet, do it now. Otherwise just expand.
		if (!this.loadedNodes [nodeId]) {
			this.LoadTree(nodeId);
		} else {
			$(this.treeName + "_ChildNodes_" + nodeId).show();
			this.expandedNodes [nodeId] = true;
		}
	}
};

// Select the given node.
eTree.prototype.Select = function (nodeId) {
	var $node = $(this.treeName + "_Link_" + nodeId);

	// If another node was selected, deselect it.
	if (this.prevSelectedNodeId != -1 && nodeId != undefined) {
		$(this.treeName + "_Link_" + this.prevSelectedNodeId).removeClassName(this.selectedNodeCssClass);
	}

	// Select the node.
	if ($node) {
		$node.removeClassName(this.nodeCssClass).addClassName(this.selectedNodeCssClass);
	}

	// Set the newly selected node.
	this.prevSelectedNodeId = nodeId;
};

// Retrieve the correct spacer icon depending on it's tree level,
// the tree level of the node and the location in the nodes in this
// tree level.
eTree.prototype.GetSpacerIcon = function (isLastSibling, x, xmax, y, ymax) {
	if (x == xmax - 1) {
		if (y == ymax - 1) {
			return "joinbottom.gif";
		} else {
			return "join.gif";
		}
	}

	if (isLastSibling) {
		return "empty.gif";
	} else {
		return "line.gif";
	}
};

// Retrieve the correct expander icon based on the bottom/expansion/leaf
// status.
eTree.prototype.GetExpanderImage = function (isBottom, isExpanded, isLeaf) {
	if (isLeaf) {
		return "leaf.gif";
	} else {
		if (isBottom) {
			if (isExpanded) {
				return "minusbottom.gif";
			} else {
				return "plusbottom.gif";
			}
		} else {
			if (isExpanded) {
				return "minus.gif";
			} else {
				return "plus.gif";
			}
		}
	}
};